@@ -364,12 +364,12 @@ instance (Serialize a, Serialize b) => Serialize (Either a b) where
 -- ByteStrings (have specially efficient instances)
 
 instance Serialize B.ByteString where
-    put bs = do put (B.length bs)
+    put bs = do put (B.length bs :: Int)
                 putByteString bs
     get    = get >>= getByteString
 
 instance Serialize L.ByteString where
-    put bs = do put (fromIntegral (L.length bs) :: Int)
+    put bs = do put (L.length bs :: Int64)
                 putLazyByteString bs
     get    = get >>= getLazyByteString
 
@@ -504,7 +504,11 @@ instance (PutSum a, PutSum b, GSerialize a, GSerialize b) => PutSum (a :+: b) wh
                              L1 x -> putSum code           sizeL x
                              R1 x -> putSum (code + sizeL) sizeR x
         where
+#if MIN_VERSION_base(4,5,0)
+          sizeL = size `unsafeShiftR` 1
+#else
           sizeL = size `shiftR` 1
+#endif
           sizeR = size - sizeL
     {-# INLINE putSum #-}
 
@@ -514,7 +518,8 @@ instance GSerialize a => PutSum (C1 c a) where
 
 ------------------------------------------------------------------------
 
-checkGetSum :: (Ord word, Bits word, GetSum f) => word -> word -> Get (f a)
+checkGetSum :: (Ord word, Num word, Bits word, GetSum f)
+            => word -> word -> Get (f a)
 checkGetSum size code | code < size = getSum code size
                       | otherwise   = fail "Unknown encoding for constructor"
 {-# INLINE checkGetSum #-}
@@ -526,7 +531,11 @@ instance (GetSum a, GetSum b, GSerialize a, GSerialize b) => GetSum (a :+: b) wh
     getSum !code !size | code < sizeL = L1 <$> getSum code           sizeL
                        | otherwise    = R1 <$> getSum (code - sizeL) sizeR
         where
+#if MIN_VERSION_base(4,5,0)
+          sizeL = size `unsafeShiftR` 1
+#else
           sizeL = size `shiftR` 1
+#endif
           sizeR = size - sizeL
     {-# INLINE getSum #-}
 
