@@ -7,7 +7,7 @@
 -- Module      : Data.Serialize.Get
 -- Copyright   : Lennart Kolmodin, Galois Inc. 2009
 -- License     : BSD3-style (see LICENSE)
--- 
+--
 -- Maintainer  : Trevor Elliott <trevor@galois.com>
 -- Stability   :
 -- Portability :
@@ -260,12 +260,18 @@ runGetLazy' m lstr = loop run (L.toChunks lstr)
   remLen c = fromIntegral (L.length lstr) - B.length c
   run str  = unGet m str Nothing (Incomplete (Just (remLen str))) failK finalK
 
-  loop _ []     =
-    (Left "Failed reading: Internal error: unexpected end of input",L.empty)
-  loop k (c:cs) = case k c of
-    Fail str   -> (Left str,L.empty)
-    Partial k' -> loop k' cs
-    Done r c'  -> (Right r,L.fromChunks (c':cs))
+  loop k chunks = case chunks of
+
+    c:cs -> case k c of
+      Fail str   -> (Left str,L.empty)
+      Partial k' -> loop k' cs
+      Done r c'  -> (Right r,L.fromChunks (c':cs))
+
+    [] -> case k B.empty of
+      Fail str   -> (Left str,L.empty)
+      Partial k' -> (Left "Failed reading: Internal error: unexpected end of input",L.empty)
+      Done r c'  -> (Right r,L.empty)
+
 {-# INLINE runGetLazy' #-}
 
 -- | Run the Get monad over a Lazy ByteString.  Note that this will not run the
@@ -350,7 +356,7 @@ uncheckedSkip n = do
 -- Fails if @ga@ fails.
 lookAhead :: Get a -> Get a
 lookAhead ga = Get $ \ s0 b0 m0 kf ks ->
-  let ks' s1 b1 = ks (s0 `B.append` bufferBytes b1) (b0 `append` b1)
+  let ks' _s1 b1 = ks (s0 `B.append` bufferBytes b1) (b0 `append` b1)
    in unGet ga s0 (Just B.empty) m0 kf ks'
 
 -- | Like 'lookAhead', but consume the input if @gma@ returns 'Just _'.
@@ -418,6 +424,7 @@ getLazyByteString n = f `fmap` getByteString (fromIntegral n)
 
 -- | Pull @n@ bytes from the input, as a strict ByteString.
 getBytes :: Int -> Get B.ByteString
+getBytes n | n < 0 = fail "getBytes: negative length requested"
 getBytes n = do
     s <- ensure n
     let consume = B.unsafeTake n s
